/*
 * File Name: radio_spi.c
 * 
 * File Contents: a set of macros to support the management of
 * EasyBee/CC2420 radio transceiver for the Flex Board, 
 * featuring dsPIC33FJ256MC710.
 * 
 * This source code is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * 
 * Main documentation: 
 * <CC2420_Data_Sheet_1_3.pdf> The CC2420 datasheet. Available at www.ti.com
 * <EasyBeeDS480.pdf> EasyBee board datasheet. Available at www.rfsolutions.co.uk
 * <Flex_ref_man_027.pdf> Flex reference Manual. Available at www.evidence.eu.com 
 * <70286A.pdf> The dsPIC33 datasheet. Available at www.microchip.com
 * 
 * Further documentation on dsPIC33x can be found at www.microchip.com 
 * 
 * Author: Gianluca Franchino.
 * Affiliation: Retis Lab. Scuola Superiore Sant'Anna. Pisa (Italy).
 * Contacts: g.franchino@sssup.it; gianluca@evidence.eu.com
 * Date: 02/27/2008.
 * 
 */
 
#include "radio_spi.h"



BYTE spi_port;

/*******************************************************************************
 * Function:        void ee_spi_init(BYTE spi_port)
 *
 * PreCondition:    None
 *
 * Input:           the spi port
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This routine initializes (opens) the spi selected by spi_port.
 *                  
 * Note:            None
 ******************************************************************************/

void ee_radio_spi_init(BYTE port)
{
		spi_port = port;
		
		if (spi_port == RADIO_SPI1) {
			
			SPI1STATbits.SPIEN = 0; /* Disable the SPI module */
		
			/*	Enable SPI internal clock 
			 *	Enable 8 bits communication mode
			 *	Enable Master Mode 
			 *	Primary Prescale 1:1
			 *	Secondary Prescale 8:1
			 *	Note: with these prescale values the port is working at 5 MHz. 
			 */
			 SPI1CON1 = 0x23;
			 
			/* Note: because the chip select pin (SS1) does not work properly, we
			 * drive this pin directly setting it as a normal I/O pin.
			 */  
			 SPI1CON2 = 0;
			 SPI1STATbits.SPIEN = 1; /* Enable the SPI module */
			 
		} else {
			SPI2STATbits.SPIEN = 0; /* Disable the SPI module */

			/*	Enable SPI internal clock 
			 *	Enable 8 bits communication mode
			 *	Enable Master Mode 
			 *	Primary Prescale 1:1
			 *	Secondary Prescale 8:1
			 *	Note: with these prescale values the port is working at 5 MHz. 
			 */

			 SPI2CON1 = 0x23;
	
			/* Note: because the chip select pin (SS1) does not work properly, we
			 * drive this pin directly setting it as a normal I/O pin.
			 */  
			 
			 SPI2CON2 = 0;
			 
			 SPI2STATbits.SPIEN = 1; /* Enable the SPI module */
			 
			
		}				
}

/*******************************************************************************
 * Function:        BYTE ee_radio_spi_put(BYTE v)
 *
 * PreCondition:    None
 *
 * Input:           the byte to send
 *
 * Output:          the byte recived during the transmission
 *
 * Side Effects:    None
 *
 * Overview:        This routine sends a byte through the spi port.
 *                  
 * Note:            None
 ******************************************************************************/
extern void (*hal_console_out)(BYTE); 

BYTE ee_radio_spi_put(BYTE v)
{
	if (spi_port == RADIO_SPI1) {
		IFS0bits.SPI1IF = 0;
   		SPI1STATbits.SPIROV = 0;
   		/* wait until the tx buffer is empty */
   		while(SPI1STATbits.SPITBF);
   		SPI1BUF = v;
   		while(!SPI1STATbits.SPIRBF);
   		SPI1STATbits.SPIROV = 0;
   		
   		return (SPI1BUF & 0xff);
	
	} else {
		IFS2bits.SPI2IF = 0;
   		SPI2STATbits.SPIROV = 0;
   		/* wait until the tx buffer is empty */
   		while(SPI2STATbits.SPITBF);
   		SPI2BUF = v;
   		//hal_console_out('A');
   		
   		while(!SPI2STATbits.SPIRBF);
   		SPI2STATbits.SPIROV = 0;
   		//hal_console_out('C');
   	   
   		return (SPI2BUF & 0xff);
   		
   	}

	return 0;
}

/*******************************************************************************
 * Function:        BYTE ee_radio_spi_get(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          the read byte
 *
 * Side Effects:    None
 *
 * Overview:        This routine reads a byte from the radio spi port.
 *                  
 * Note:            None
 ******************************************************************************/

BYTE ee_radio_spi_get(void)
{

	if (spi_port == RADIO_SPI1) {
   		/* wait until the tx buffer is empty */
   		while(SPI1STATbits.SPITBF);
   		SPI1BUF = 0x00;
   		while(!SPI1STATbits.SPIRBF);
   		SPI1STATbits.SPIROV = 0;
   		
   		return (SPI1BUF & 0xff);
	
	} else {
   		/* wait until the tx buffer is empty */
   		while(SPI2STATbits.SPITBF);
   		SPI2BUF = 0x00;
   		while(!SPI2STATbits.SPIRBF);
   		SPI2STATbits.SPIROV = 0;

   		return (SPI2BUF & 0xff);

   	}   
	return 0; 
} 
